# Veronica Torralba (CMCC)
# 15th November 2022
# Medcof training - AEMET

# cleaning:
rm(list=ls())

# To run this example several libraries are required
# install.packages('CSTools', 'zeallot', 's2dv','ClimProjDiags')

# load libraries:
library(CSTools)
library(s2dv)
library(ClimProjDiags)
library(zeallot)

# define paths:
path <- '/home/tr100/CST_WeatherRegimes/data_storage'
exp_path <- list(
  list(name = 'cmcc',
       path = paste0(
         path,
         '/exp/cmcc/sps35/daily_mean/$VAR_NAME$/$VAR_NAME$_$YEAR$$MONTH$01.nc')),
  list(name = 'ecmwf',
       path = paste0(
         path,
         '/exp/ecmwf/seas5/daily_mean/$VAR_NAME$/$VAR_NAME$_$START_DATE$.nc')))

obs_path <- list(
  list(name = 'era5',
       path = paste0(
         path,
         '/recon/ecmwf/era5/daily_mean/$VAR_NAME$/$VAR_NAME$_$YEAR$$MONTH$.nc')))


# The seasonal forecasts from the two systems (CMCC_SPS35, ECMWF_SEAS5) 
# and the observational reference (ERA5) are stored in two objects: exp and obs
# CST_Load provides corresponding forecasts and observations
# for a common  period and grid 
# The selected variable is the mean sea level pressure

c(exp,obs) %<-%  CST_Load(var = 'slp',
                          exp = exp_path,
                          obs = obs_path,
                          sdates = paste0(1993:2009,'0501'),
                          storefreq ='daily',
                          nmember = 10,
                          leadtimemin = 1, leadtimemax = 31,
                          latmin = 27, latmax = 48,
                          lonmin = -12, lonmax = 40,
                          output = 'lonlat',nprocs=1)

# Daily anomalies
c(ano_exp, ano_obs) %<-% CST_Anomaly(exp = exp, obs = obs, filter_span = 1)

# Check anomalies' dimensions
dim(ano_exp$data)
dim(ano_obs$data)

# Compute the weather regimes (from the daily observed anomalies)
WR_obs <- CST_WeatherRegimes(data = ano_obs, EOFs = FALSE, ncenters = 4)

# Plot the most recurrent observed patterns with their climatological frequencies
clim_freq <- apply(WR_obs$statistics$frequency,c(2,3,4),mean)

PlotLayout(PlotEquiMap, c(1, 2), lon = obs$lon, lat = obs$lat,
           var = WR_obs$data / 100,
           titles = paste0(paste0('Cluster ', 1:4), ' (', paste0('freq = ',
                                                                 round(clim_freq,1 ), '%'),' )'),
           filled.continents = FALSE,
           axelab = FALSE, draw_separators = TRUE, subsampleg = 1,
           brks = seq(-16, 16, by = 2),
           bar_extra_labels = c(2, 0, 0, 0),
           fileout = '~/CST_WeatherRegimes/observed_regimes.png')


# Compute the weather regimes (from the daily seasonal forecast anomalies and 
# the observed regimes as a reference)

# We will compute the regimes separately for each model. 
ano_exp_cmcc <- ano_exp_ecmwf <- ano_exp
ano_exp_cmcc$data <- Subset(ano_exp$data,along='dataset',indices=1)
ano_exp_ecmwf$data <- Subset(ano_exp$data,along='dataset',indices=2)

WR_exp_cmcc <- CST_RegimesAssign(data = ano_exp_cmcc, ref_maps = WR_obs, 
                                 method = 'distance',
                                 composite = TRUE, memb = T)

WR_exp_ecmwf <- CST_RegimesAssign(data = ano_exp_ecmwf, ref_maps = WR_obs, 
                                  method = 'distance',
                                  composite = TRUE, memb = T)

# Plot the most recurrent predicted patterns with their climatological
# frequencies

PlotLayout(
  PlotEquiMap,
  c(1, 2),
  lon = exp$lon,
  lat = exp$lat,
  var = WR_exp_cmcc$data / 100,
  titles = paste0(paste0('Cluster ', 1:4), ' (', paste0(
    'freq = ', round(WR_exp_cmcc$statistics$frequency, 1),'%'
  ), ' )'),
  filled.continents = FALSE,
  axelab = FALSE,
  draw_separators = TRUE,
  subsampleg = 1,
  toptitle='CMCC',   
  brks = seq(-16, 16, by = 2),
  bar_extra_labels = c(2, 0, 0, 0),
  fileout = '~/CST_WeatherRegimes/predicted_regimes_cmcc.png')

PlotLayout(
  PlotEquiMap,
  c(1, 2),
  lon = exp$lon,
  lat = exp$lat,
  var = WR_exp_ecmwf$data / 100,
  toptitle='ECMWF',   
  titles = paste0(paste0('Cluster ', 1:4), ' (', paste0(
    'freq = ', round(WR_exp_ecmwf$statistics$frequency, 1), '%'
  ), ' )'),
  filled.continents = FALSE,
  axelab = FALSE,
  draw_separators = TRUE,
  subsampleg = 1,
  brks = seq(-16, 16, by = 2),
  bar_extra_labels = c(2, 0, 0, 0),
  fileout = '~/CST_WeatherRegimes/predicted_regimes_ecwmf.png')
